/**
 * PROIECT PM - Control-bt
 * 
 * Chip type: ATmega328 + enc28j60 + parani esd100
 * Conectare: 
 * 		PC(s) - placuta: Ethernet (static ip, HTTP)
 * 		telefon - placuta: Bluetooth
 * Further work:
 * 		* use UDP for broadcast
 * 		* some authorization to prevent hijacking
 * 
 * Authors: Mihai Dumitrache <mihai [at] wip.ro>
 * 			Alex Eftimie <alex [at] eftimie.ro>
 * Copyright: GPL V2
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <avr/pgmspace.h>
#include "ip_arp_udp_tcp.h"
#include "enc28j60.h"
#include "timeout.h"
#include "avr_compat.h"
#include "net.h"
#include "seriala.h"

// Please modify the following two lines.
// Attention! MAC and IP have to be unique in your local area network.
// You can not have the same numbers in two devices. If a conflict appears
// change it!
static uint8_t mymac[6]     =   {0x00, 0x15, 0xC5, 0xB4, 0x91, 0x58};
static uint8_t myip[4]      =   {192, 168, 1, 99};
#define MY_IP_STR               "192.168.1.99"

// UDP stuff
//static uint8_t remotemac[6] = 	{0x00, 0x19, 0x7d, 0x60, 0x0F, 0xe6};
//static uint8_t remoteip[4] 	= 	{192, 168, 1, 255};
//static uint16_t port = 33838;

uint16_t send_msg = 0, buf_primit = 0;
/* buffer in care primesc mesajele */
unsigned char message[65];

// Your IP.
uint8_t yip[4];

// Base url (you can put a DNS name instead of an IP address if you have
// a DNS server (baseurl must end in "/").
static char baseurl[]       =   "http://192.168.1.99/";

// Listen port for tcp/www (max range 1-254).
static uint16_t mywwwport   =   80;

#define BUFFER_SIZE             800

unsigned char buf[BUFFER_SIZE + 1];
uint8_t page = 1;
uint8_t num = 0;

uint16_t milisec = 0;
uint8_t secunde = 0, minute = 0, ore = 0;

int8_t analyse_get_url(char *str)
{
        uint8_t i = 0;

        while (*str && i < 10 && *str > ',' && *str < '{') {
                if (*str == '/') {
                        page = 1;
                        ++str;
                        break;
                } else if (*str == '?') {
                        num = 0;
                        page = 1;
                        ++str;
                        break;
                }
                ++i;
                ++str;
        }

        while (*str) {
                if (*str == 'c') {
                        ++str;
                        page = 0;
                } else if (*str == 'f' || *str == 's') {
                        page = 0;
                        break;
                } else if (*str == 't') {
                        page = 2;
                        break;
                } else if (*str == 'l') {
                        page = 3;
                        break;
                } else if (*str == 'b') {
                        page = 4;
                        break;
                } else if (*str == 'h') {
                        page = 5;
                        break;
                } else if (*str == 'd') {
                        page = 6;
                        break;
				} else if (*str == 'x') {
						page = 7;
						break;
                } else {
                        break;
                }

                num |= (1 << ((*str-0x30)-1));
                str += 4;
        }

        return num;
}

uint16_t print_webpage(uint8_t *buf, uint8_t on_off)
{
        char buffer[15];
        uint16_t plen;

        plen = fill_tcp_data_p(buf, 0, PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
        if (page == 0) {
                plen = fill_tcp_data_p(buf, plen, PSTR("<HTML><BODY bgcolor=LightCyan>\n"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<FORM action=\"http://"MY_IP_STR"\" method=\"get\">"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c1\" value=\"1\""));
                if (on_off & 0x01) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED0<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c2\" value=\"1\""));
                if (on_off & 0x02) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED1<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c3\" value=\"1\""));
                if (on_off & 0x04) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED2<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c4\" value=\"1\""));
                if (on_off & 0x08) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED3<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c5\" value=\"1\""));
                if (on_off & 0x10) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED4<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c6\" value=\"1\""));
                if (on_off & 0x20) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED5<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c7\" value=\"1\""));
                if (on_off & 0x40) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED6<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<input type=\"checkbox\" name=\"c8\" value=\"1\""));
                if (on_off & 0x80) plen = fill_tcp_data_p(buf, plen, PSTR(" checked"));
                plen = fill_tcp_data_p(buf, plen, PSTR(">LED7<br>"));

                plen = fill_tcp_data_p(buf, plen, PSTR("<br><INPUT type=\"submit\" name=\"submit\" value=\"OK\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("</FORM>\n</BODY>\n</HTML>\n"));
        } else if (page == 2) {
                plen = fill_tcp_data_p(buf, plen, PSTR("<html><head><META HTTP-EQUIV=REFRESH CONTENT=10></head>"
                                                       "<body bgcolor=aqua style=\"text-align: center\">Time up<br>"));
                sprintf(buffer, "%02d:%02d:%02d", ore, minute, secunde);
                plen = fill_tcp_data(buf, plen, buffer);
                plen = fill_tcp_data_p(buf, plen, PSTR("</body></html>"));
        } else if (page == 3) {
                plen = fill_tcp_data_p(buf, plen, PSTR("<html><body><font color=navy><span style=\"font-family: Arial\">Menu<br><br><hr>"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<a href =\"http://"MY_IP_STR"/sec/h.htm\" target =\"content\">Home</a><br><hr>"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<a href =\"http://"MY_IP_STR"/sec/f.htm\" target =\"content\">Leds</a><br><hr>"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<a href =\"http://"MY_IP_STR"/sec/d.htm\" target =\"content\">Time</a><hr>"));
                plen = fill_tcp_data_p(buf, plen, PSTR("</font></body></html>"));
        } else if (page == 4) {
                plen = fill_tcp_data_p(buf, plen, PSTR("<html><body bgcolor=navy  style=\"text-align: right\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<font color=white size=20><span style=\"font-family: Century Gothic\">webServer1.0</span></font>"));
                plen = fill_tcp_data_p(buf, plen, PSTR("</body></html>"));
        } else if (page == 5) {
                plen = fill_tcp_data_p(buf, plen, PSTR("<html><body bgcolor=LightCyan  style=\"text-align: center\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<font color=navy size=12><span style=\"font-family: Arial\"><br><br>Server is up and running.<br>Server IP: "));
                sprintf(buffer, "%d.%d.%d.%d", myip[0], myip[1], myip[2], myip[3]);
                plen = fill_tcp_data(buf, plen, buffer);
                plen = fill_tcp_data_p(buf, plen, PSTR("<br>Your IP: "));
                sprintf(buffer, "%d.%d.%d.%d", yip[0], yip[1], yip[2], yip[3]);
                plen = fill_tcp_data(buf, plen, buffer);
                plen = fill_tcp_data_p(buf, plen, PSTR("</span></font></body></html>"));
        } else if (page == 6) {
                plen = fill_tcp_data_p(buf, plen, PSTR("<HTML><script language=\"JavaScript\"> var clockID = 0;"));
                plen = fill_tcp_data_p(buf, plen, PSTR("function UpdateClock(){if(clockID){clearTimeout(clockID);clockID  = 0;} var tDate = new Date();"));
                plen = fill_tcp_data_p(buf, plen, PSTR(" document.theClock.theTime.value = \"\"+ tDate.getHours()+\":\"+tDate.getMinutes()+\":\"+tDate.getSeconds();"));
                plen = fill_tcp_data_p(buf, plen, PSTR(" clockID = setTimeout(\"UpdateClock()\", 1000);}"));
                plen = fill_tcp_data_p(buf, plen, PSTR("function StartClock() {clockID = setTimeout(\"UpdateClock()\", 500);}"));
                plen = fill_tcp_data_p(buf, plen, PSTR("function KillClock() {if(clockID){clearTimeout(clockID);clockID  = 0;}}"));
                plen = fill_tcp_data_p(buf, plen, PSTR("</script><body onload=\"StartClock()\" onunload=\"KillClock()\" bgcolor=LightCyan><center><form name=\"theClock\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("The time is: <input type=text name=\"theTime\" style=\"font-family: Arial Black\" style=\"font-size=15\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("</form></center></body></HTML>"));
        } else if (page == 7) {
				if (send_msg > 0) {
					sprintf(buffer, "%s", message);
					send_msg-- ;
				}
				else {
					sprintf(buffer, "%s ", DEFAULT_MSG);					
				}
                plen = fill_tcp_data(buf, plen, buffer);
		}
		else {
                plen = fill_tcp_data_p(buf, plen, PSTR("<HTML><HEAD><TITLE>WebServer</TITLE></HEAD> "));
                plen = fill_tcp_data_p(buf, plen, PSTR("<FRAMESET cols=\"10%,90%\"><FRAMESET rows=\"90%,10%\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<FRAME noresize name=\"links\" src=\"http://"MY_IP_STR"/sec/l.htm\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<FRAME noresize name=\"time\" src=\"http://"MY_IP_STR"/sec/t.htm\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("</FRAMESET><FRAMESET rows=\"15%,85%\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<FRAME noresize name=\"banner\" src=\"http://"MY_IP_STR"/sec/b.htm\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("<FRAME noresize name=\"content\" src=\"http://"MY_IP_STR"/sec/h.htm\">"));
                plen = fill_tcp_data_p(buf, plen, PSTR("</FRAMESET></FRAMESET></HTML>"));
        }

        return plen;
}

// Init timer2 to generate an interrupt every milisecond.
void initTimer2(void)
{
        TCCR2 = _BV(WGM21) | _BV(CS20) | _BV(CS21) | _BV(CS22);
		OCR2 = 15;
		TIMSK = _BV(OCIE2);
}

// Output compare interrupt handler for timer2.
SIGNAL(SIG_OUTPUT_COMPARE2)
{
        milisec++;
		if(milisec == 1000)
		{
			milisec = 0;
			secunde++;
		}
		if(secunde == 60)
		{
			secunde = 0;
			minute++;
		}
		if(minute == 60)
		{
			minute = 0;
			ore++;
		}
		if(ore == 24)
			ore = 0;
}

SIGNAL(SIG_UART_RECV)
{	
	char c;	
	c = uart_getchar();
	DDRD = (1<<PD6);
	
	if (c == '@') {
		buf_primit = 0;
	}
	if (c != 0 && c != '\n' && c != '@') {
		message[buf_primit++] = c;
	}
	if (buf_primit == 8 || c == ' ') {
		send_msg ++;
		message[buf_primit] = 0;
		buf_primit = 0;
	}
}

int main(void) {
        uint16_t plen;
        uint16_t dat_p;
        uint8_t i = 0;
        int8_t cmd;

		delay_ms(10);
        
		usart_init();
        sei();
		send_msg = 0;
		
        // Enable PB2, reset as output.
        DDRB |= (1<<PB2);

        // Set output to gnd, reset the ethernet chip.
        PORTB &= ~(1<<PB2);
        delay_ms(10);

        // Set output to Vcc, reset inactive.
        PORTB |= (1<<PB2);
        delay_ms(200);

        // Initialize enc28j60.
        enc28j60Init(mymac);
        delay_ms(20);

        enc28j60PhyWrite(PHLCON, 0x476);
        delay_ms(20);

        i = 1;

        // Init the ethernet/ip layer.
        init_ip_arp_udp_tcp(mymac, myip, mywwwport);
		
		send_msg = 0;
        while (1) {
				
                // Get the next new packet.
                plen = enc28j60PacketReceive(BUFFER_SIZE, buf);
				
				// plen will be different than zero if there is a valid
                // packet (without crc error).
                if (plen == 0) {
                        continue;
                }

                // Arp is broadcast if unknown but a host may also
                // verify the mac address by sending it to
                // a unicast address.
                if (eth_type_is_arp_and_my_ip(buf, plen)) {
                        make_arp_answer_from_request(buf);
                        continue;
                }

                // Check if ip packets are for us.
                if (eth_type_is_ip_and_my_ip(buf,plen) == 0) {
                        continue;
                }
	
                if (buf[IP_PROTO_P] == IP_PROTO_ICMP_V && buf[ICMP_TYPE_P] == ICMP_TYPE_ECHOREQUEST_V) {
                        // A ping packet, let's send pong.
						// Chestie alex
						make_echo_reply_from_request(buf,plen);
                        continue;
                }

                // TCP port www start, compare only the lower byte.
                if (buf[IP_PROTO_P] == IP_PROTO_TCP_V && buf[TCP_DST_PORT_H_P] == 0 && buf[TCP_DST_PORT_L_P] == mywwwport) {
                        // Read source IP.
                        yip[0] = buf[26];
                        yip[1] = buf[27];
                        yip[2] = buf[28];
                        yip[3] = buf[29];

                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V) {
                                make_tcp_synack_from_syn(buf);
                                // make_tcp_synack_from_syn does already send the syn,ack.
                                continue;
                        }

                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V) {
                                init_len_info(buf);

                                // We can possibly have no data, just ack.
                                dat_p = get_tcp_data_pointer();
                                if (dat_p == 0) {
                                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V) {
                                                // Finack, answer with ack.
                                                make_tcp_ack_from_any(buf);
                                        }

                                        // Just an ack with no data, wait for next packet.
                                        continue;
                                }

                                if (strncmp("GET ", (char *)&(buf[dat_p]), 4) != 0) {
                                        // Head, post and other methods.
                                        // For possible status codes see:
                                        // http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
                                        plen = fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n<h1>200 OK</h1>"));
                                        goto SENDTCP;
                                }
                                if (strncmp("/ ", (char *)&(buf[dat_p + 4]), 2) == 0) {
                                        plen=fill_tcp_data_p(buf, 0, PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
                                        plen=fill_tcp_data_p(buf, plen, PSTR("<p>Usage: "));
                                        plen=fill_tcp_data(buf, plen, baseurl);
                                        plen=fill_tcp_data_p(buf, plen, PSTR("/</p>"));
                                        goto SENDTCP;
                                }

                                cmd = analyse_get_url((char *)&(buf[dat_p + 5]));

                                plen = print_webpage(buf,cmd);
SENDTCP:
								make_tcp_ack_from_any(buf);  // send ack for http get
								make_tcp_ack_with_data(buf,plen);  // send data
								continue;
                        }
                }
        }

        return 0;
}

